define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojox/html/entities",

     "dijit/Destroyable",
// epi
    "epi/dependency",
    "epi-cms/contentediting/_ContentEditingNotification",
    "epi-cms/core/ContentReference",

    // template
    "dojo/text!./ProjectNotification.html",

// resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.notifications"
],

function (
// dojo
    declare,
    lang,
    entities,
    Destroyable,
// epi
    dependency,
    _ContentEditingNotification,
    ContentReference,

//template
    template,

// resources
    resources
) {

    return declare([_ContentEditingNotification, Destroyable], {
        // summary:
        //      Show notification if the content is part of a project
        // tags:
        //      internal

        // projectService: [readonly] ProjectService
        //      A service for interacting with projects
        projectService: null,

        _currentContentLink: null,

        postscript: function() {
            this.inherited(arguments);

            this.projectService = this.projectService || dependency.resolve("epi.cms.ProjectService");

            this.own(
                this.projectService.on("items-added", lang.hitch(this, this._onItemsAddedOrRemoved)),
                this.projectService.on("items-removed", lang.hitch(this, this._onItemsAddedOrRemoved))
            );
        },

        _onExecuteSuccess: function(content) {
            if(!content || !content.contentLink) {
                this._currentContentLink = null;
                return;
            }
            this._currentContentLink = content.contentLink;

            this._updateNotification(content.contentLink);
        },

        _onItemsAddedOrRemoved: function(data) {
            if(!this._currentContentLink) {
                return;
            }

            var hasContentLinkEqualTo = function(currentContentLink) {
                return function(item) {
                    return ContentReference.compareIgnoreVersion(currentContentLink, item.contentLink);
                };
            };

            if (data.items.some(hasContentLinkEqualTo(this._currentContentLink))) {
                this._updateNotification(this._currentContentLink);
            }
        },

        _updateNotification: function(contentLink) {
            var self = this;

            this.projectService.getProjectsForContent(contentLink).then(function(projects) {
                if(projects.length === 0) {
                    self._setNotification(null);
                    return;
                }

                var projectNames = projects.map(function(project) {
                    return entities.encode(project.name);
                }).join(", ");

                var content = lang.replace(template, {
                    partOfProject: resources.partofproject,
                    projects: projectNames
                });

                self._setNotification({
                    content: content
                });
            });
        }
    });
});
