define([
    "dojo/_base/declare",

    "./_ProjectCommand",

    "../ProjectSchedulingDialog",
    "../ProjectSchedulingDialogViewModel",
//Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.command.scheduleproject"
], function (declare,
             _ProjectCommand,
             ProjectSchedulingDialog,
             ProjectSchedulingDialogViewModel,
             res
             ) {

    return declare([_ProjectCommand], {
        // summary:
        //      Command for setting scheduled publishing on a project.
        // tags:
        //      internal

        // category: [readonly] String
        //      A category which provides a hint about how the command could be displayed.
        category: "publishmenu",

        // label: [public] String
        //      The action text of the command to be used in visual elements.
        label: res.label,

        // iconClass: [public] String
        //      The icon class of the command to be used in visual elements.
        iconClass: "epi-iconClock",

        _execute: function () {
            // summary:
            //      Executes this command assuming canExecute has been checked.
            // tags:
            //      protected

            var model = this.model;
            var project = model.selectedProject;

            var viewModel = new ProjectSchedulingDialogViewModel({
                title: project.name,
                query: {projectId: project.id, scheduled: true}
            });

            var dialog = new ProjectSchedulingDialog({model: viewModel});

            dialog.on("execute", function () {
                model.publishProject(project.id, viewModel.dateValue);
            });

            dialog.show();
        },

        _setCanExecute: function () {
            var selectedProject = this.model.selectedProject,
                status = selectedProject && selectedProject.status;

            this.set("canExecute", !!selectedProject && selectedProject.canBePublished);

            // Check delay published as well
            this.set("isAvailable", status !== "published" && status !== "delayedpublished");
        }
    });
});
